package com.zswl.cloud.springBtach.server.core.api.qiuxiang;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.github.microservice.auth.security.helper.AuthHelper;
import com.zswl.cloud.springBatch.client.model.qiuxiang.OrderRequest;
import com.zswl.cloud.springBtach.server.core.dao.ApiLogDao;
import com.zswl.cloud.springBtach.server.core.domain.ApiLog;
import com.zswl.cloud.springBtach.server.core.helper.RedisHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Map;

@Component
public class QiuXiangApi {

    @Value("${qxdj.url}")
    private String BASE_URL;

    @Value("${qxdj.appid}")
    private String APPID;

    @Value("${qxdj.secret}")
    private String SECRET;

    @Autowired
    private ApiLogDao apiLogDao;

    @Autowired
    AuthHelper authHelper;

    private final String qiuxiang_token = "qiuxiang_token";
    private final String qxUserToken = "qxUser_token";
    private final String system = "qiuxiang";

    @Autowired
    RedisHelper redisHelper;

    /**
     * 获取接口调用凭证
     *
     * @return token
     */
    public String getPlatformToken() {
        Object token = redisHelper.getValue(qiuxiang_token);

        if (ObjectUtil.isEmpty(token)) {
            ApiLog apiLog = new ApiLog();
            String url = BASE_URL + "/token";
            HttpRequest httpRequest = HttpRequest.get(url);
            Map<String, String> appid = Map.of("appid", APPID, "secret", SECRET);
            String result = httpRequest
                    .form("appid", APPID)
                    .formStr(appid)
                    .execute().body();
            apiLog.setService("/token");
            apiLog.setSystem(system);
            apiLog.setRequest(appid.toString());
            apiLog.setResponse(result);
            apiLogDao.save(apiLog);
            JSONObject json = new JSONObject(result);
            String newToken = json.getStr("token");
            redisHelper.cacheValue(qiuxiang_token, newToken, 7000);
            return newToken;
        }

        return String.valueOf(token);
    }

    /**
     * 获取接口调用权限
     *
     * @param token 接口调用凭证
     * @return permissions
     */
    public String getPermissions() {
        String url = BASE_URL + "/permissions";
        String result = HttpRequest.get(url)
                .header("Authorization", "Bearer " + getPlatformToken())
                .execute().body();

        JSONObject json = new JSONObject(result);
        return json.toString();
    }

    /**
     * 用户登录
     *
     * @param phone 用户手机号码
     * @return 登录信息，包含uid和新的token
     */
    public String loginUser(String phone) {

        Object userToken = redisHelper.getValue(qxUserToken + phone);
        if (ObjectUtil.isEmpty(userToken)) {
            ApiLog apiLog = new ApiLog();
            String url = BASE_URL + "/user/login";
            Map<String, String> parm = Map.of("phone", phone);
            String result = HttpRequest.get(url)
                    .header("Token", getPlatformToken())
                    .formStr(parm)
                    .execute().body();

            apiLog.setSystem(system);
            apiLog.setService("/user/login");
            apiLog.setRequest(phone.toString());
            apiLog.setResponse(result);
            apiLogDao.save(apiLog);

            JSONObject json = new JSONObject(result);
            String token = json.getStr("token");
            redisHelper.cacheValue(qxUserToken + phone, token, 2591000);
            return token;
        }

        return String.valueOf(userToken);
    }

    /**
     * 用户下单
     *
     * @param token        用户登录后的token
     * @param orderRequest 下单请求参数
     * @return 订单信息，包含订单ID
     */
    public JSONObject placeOrder(OrderRequest orderRequest) {
        String url = "/user/order";

        JSONObject body = new JSONObject();
        body.put("adcode", orderRequest.getAdcode());
        body.put("originName", orderRequest.getOriginName());
        body.put("originLocation", orderRequest.getOriginLocation());
        body.put("originLongitude", orderRequest.getOriginLongitude());
        body.put("originLatitude", orderRequest.getOriginLatitude());

        if (orderRequest.getDestinationName() != null && orderRequest.getDestinationLocation() != null) {
            body.put("destinationName", orderRequest.getDestinationName());
            body.put("destinationLocation", orderRequest.getDestinationLocation());
            body.put("destinationLongitude", orderRequest.getDestinationLongitude());
            body.put("destinationLatitude", orderRequest.getDestinationLatitude());
        }

        String result = postRequest(url, body.toString());
        return new JSONObject(result);
    }

    /**
     * 用户订单列表
     *
     * @param userToken
     * @param lastOrderId
     * @param count
     * @return
     */
    public JSONArray orderList(String lastOrderId, String count) {
        String url = "/user/order/list";
        Map<String, String> orderId1 = Map.of("lastOrderId", lastOrderId, "count",count);
        String request = getRequest(url, orderId1);


        return JSONUtil.parseArray(request);
    }

    /**
     * 订单详情
     *
     * @param userToken
     * @param orderId
     * @return
     */
    public JSONObject orderDelit(String orderId) {
        String url = "/user/order";
        Map<String, String> orderId1 = Map.of("orderId", orderId);
        String result = getRequest(url, orderId1);

        JSONObject json = new JSONObject(result);
        return json;
    }


    /**
     * 取消订单
     *
     * @param token
     * @param orderId
     * @param reason
     * @return
     */
    public JSONObject cancelOrder(String orderId, String reason) {
        String url = "/user/order";
        Map<String, String> params = Map.of("orderId", orderId, "reason", reason);
        String result = deleteRequest(url, JSONUtil.toJsonStr(params));

        JSONObject json = new JSONObject(result);
        return json;
    }

    /**
     * 获取司机实时位置
     *
     * @param token
     * @param orderId
     * @return
     */
    public JSONObject driverLocation( String orderId) {
        String url = "/user/order/driver/location";
        String request = getRequest(url, Map.of("orderId", orderId));
        return new JSONObject(request);
    }

    /**
     * 获取司机行驶轨迹
     *
     * @param token
     * @param orderId
     * @return
     */
    public JSONArray driverLocationLog(String orderId) {
        String url = "/user/order/driver/location/log";
        String request = getRequest(url, Map.of("orderId", orderId));
        return JSONUtil.parseArray(request);
    }

    private String getRequest(String url, Map<String, String> params) {
        String phone = authHelper.getCurrentUser().getPhone();
        String token = loginUser(phone);
        String result = HttpRequest.get(BASE_URL + url)
                .header("Token", token)
                .formStr(params)
                .execute().body();
        ApiLog apiLog = new ApiLog();
        apiLog.setSystem(system);
        apiLog.setService(url);
        apiLog.setRequest(JSONUtil.toJsonStr(params));
        apiLog.setResponse(result);
        apiLogDao.save(apiLog);
        return result;
    }

    private String deleteRequest(String url, String jsonString) {
        String phone = authHelper.getCurrentUser().getPhone();
        String token = loginUser(phone);
        String result = HttpRequest.delete(BASE_URL + url)
                .header("Token", token)
                .body(jsonString)
                .execute().body();
        ApiLog apiLog = new ApiLog();
        apiLog.setSystem(system);
        apiLog.setService(url);
        apiLog.setRequest(jsonString);
        apiLog.setResponse(result);
        apiLogDao.save(apiLog);

        return result;
    }

    private String postRequest(String url, String jsonString) {
        String phone = authHelper.getCurrentUser().getPhone();
        String token = loginUser(phone);
        String result = HttpRequest.post(url)
                .header("Token", token)
                .body(jsonString)
                .execute().body();
        ApiLog apiLog = new ApiLog();
        apiLog.setSystem(system);
        apiLog.setService(url);
        apiLog.setRequest(jsonString);
        apiLog.setResponse(result);
        apiLogDao.save(apiLog);

        return result;

    }
}
